
package org.mule.modules.jive.config.spring;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.jive.config.ExecuteMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class ExecuteDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public ExecuteDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(ExecuteMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        if (element.hasAttribute("operation")) {
            builder.addPropertyValue("operation", element.getAttribute("operation"));
        }
        Element entityListElement = null;
        entityListElement = DomUtils.getChildElementByTagName(element, "entity");
        List<Element> entityListChilds = null;
        if (entityListElement!= null) {
            String entityRef = entityListElement.getAttribute("ref");
            if ((entityRef!= null)&&(!StringUtils.isBlank(entityRef))) {
                if ((!entityRef.startsWith(patternInfo.getPrefix()))&&(!entityRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("entity", new RuntimeBeanReference(entityRef));
                } else {
                    builder.addPropertyValue("entity", entityRef);
                }
            } else {
                ManagedMap entity = new ManagedMap();
                entityListChilds = DomUtils.getChildElementsByTagName(entityListElement, "entity");
                if (entityListChilds!= null) {
                    if (entityListChilds.size() == 0) {
                        entityListChilds = DomUtils.getChildElements(entityListElement);
                    }
                    for (Element entityChild: entityListChilds) {
                        String entityValueRef = entityChild.getAttribute("value-ref");
                        String entityKeyRef = entityChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((entityValueRef!= null)&&(!StringUtils.isBlank(entityValueRef))) {
                            valueObject = new RuntimeBeanReference(entityValueRef);
                        } else {
                            valueObject = entityChild.getTextContent();
                        }
                        if ((entityKeyRef!= null)&&(!StringUtils.isBlank(entityKeyRef))) {
                            keyObject = new RuntimeBeanReference(entityKeyRef);
                        } else {
                            keyObject = entityChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = entityChild.getTagName();
                        }
                        entity.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("entity", entity);
            }
        }
        if ((element.getAttribute("id")!= null)&&(!StringUtils.isBlank(element.getAttribute("id")))) {
            builder.addPropertyValue("id", element.getAttribute("id"));
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
